@@ -136,7 +136,7 @@ impl<'a> SMBiosProcessorInformation<'a> {
             .map(|raw| ProcessorUpgradeData::from(raw))
     }
 
-    /// Handle of a [SMBiosCacheInformation] structure that
+    /// Handle of a [super::SMBiosCacheInformation] structure that
     /// defines the attributes of the primary (Level 1)
     /// cache for this processor
     ///
@@ -149,7 +149,7 @@ impl<'a> SMBiosProcessorInformation<'a> {
         self.parts.get_field_handle(0x1A)
     }
 
-    /// Handle of a [SMBiosCacheInformation] structure that
+    /// Handle of a [super::SMBiosCacheInformation] structure that
     /// defines the attributes of the primary (Level 2)
     /// cache for this processor
     ///
@@ -162,7 +162,7 @@ impl<'a> SMBiosProcessorInformation<'a> {
         self.parts.get_field_handle(0x1C)
     }
 
-    /// Handle of a [SMBiosCacheInformation] structure that
+    /// Handle of a [super::SMBiosCacheInformation] structure that
     /// defines the attributes of the primary (Level 3)
     /// cache for this processor
     ///
@@ -1214,19 +1214,19 @@ impl From<u16> for ProcessorFamily {
             0xFE => ProcessorFamily::SeeProcessorFamily2,
             0x100 => ProcessorFamily::ARMv7,
             0x101 => ProcessorFamily::ARMv8,
-            0x102 => ProcessorFamily::SH3,
-            0x103 => ProcessorFamily::SH4,
-            0x104 => ProcessorFamily::ARM,
-            0x105 => ProcessorFamily::StrongARM,
-            0x106 => ProcessorFamily::Cyrix6x86,
-            0x107 => ProcessorFamily::MediaGX,
-            0x108 => ProcessorFamily::MII,
-            0x109 => ProcessorFamily::WinChip,
-            0x10A => ProcessorFamily::DSP,
-            0x10B => ProcessorFamily::VideoProcessor,
-            0x10C => ProcessorFamily::RISCVRV32,
-            0x10D => ProcessorFamily::RISCVRV64,
-            0x10E => ProcessorFamily::RISCVRV128,
+            0x104 => ProcessorFamily::SH3,
+            0x105 => ProcessorFamily::SH4,
+            0x118 => ProcessorFamily::ARM,
+            0x119 => ProcessorFamily::StrongARM,
+            0x12C => ProcessorFamily::Cyrix6x86,
+            0x12D => ProcessorFamily::MediaGX,
+            0x12E => ProcessorFamily::MII,
+            0x140 => ProcessorFamily::WinChip,
+            0x15E => ProcessorFamily::DSP,
+            0x1F4 => ProcessorFamily::VideoProcessor,
+            0x200 => ProcessorFamily::RISCVRV32,
+            0x201 => ProcessorFamily::RISCVRV64,
+            0x202 => ProcessorFamily::RISCVRV128,
             _ => ProcessorFamily::None,
         }
     }
